//
//  media-type-pie-chart.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-17.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//


function MediaTypePieChart(chartDiv, svg, width, height) {
    this.chartDiv = chartDiv;
    chartDiv.classed("pie-chart media-type-pie-chart", true);

    var radius = Math.min(width, height) / 2;
    this.arc = d3.svg.arc().innerRadius(0).outerRadius(radius);
    this.labelArc = d3.svg.arc().innerRadius(16).outerRadius(radius);

    this.pieLayout = d3.layout.pie().value(function(d) { return d.count; }).sort(null);

    this.color = d3.scale.category10();

    this.segmentContainer =
        svg
        .append("g")
        .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");
    this.labelContainer =
        svg
        .append("g")
        .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");
}


MediaTypePieChart.prototype.update = function() {
    var self = this;
    var filterMediaType = dataProvider.filterMediaType();
    var unfilteredMediaTypeCounts = JSON.parse(dataProvider.unfilteredMediaTypeCounts());

    var countOfMediaWithTypes = 0;
    for (var i = 0; i < unfilteredMediaTypeCounts.length; i++)
        countOfMediaWithTypes += unfilteredMediaTypeCounts[i].count;
    if (!countOfMediaWithTypes) {
        this.segmentContainer
            .selectAll("g")
            .remove();
        this.labelContainer
            .selectAll("text")
            .remove();
        return;
    }

    function segmentPathName(d) {
        return "segment-path-" + d.data.mediaType.toLowerCase();
    }
    function clipPathName(d) {
        return "clip-path-" + d.data.mediaType.toLowerCase();
    }

    var data = this.pieLayout(unfilteredMediaTypeCounts);

    var segments =
        this.segmentContainer
        .selectAll("g")
        .data(data);


    var labels =
        this.labelContainer
        .selectAll("text")
        .data(data);

    // insert
    var enteredSegments =
        segments
        .enter()
        .append("g")
        .classed("clickable", true)
        .on("click", function(d) {
            // this is only bound once, so must check current type, not captured variable
            if (dataProvider.filterMediaType() != d.data.mediaType)
                dataProvider.setFilterMediaType(d.data.mediaType);
            else
                dataProvider.setFilterMediaType(null);
        });

    enteredSegments
        .append("path")
        .attr("id", segmentPathName)
        .attr("fill", function(d, i) { return self.color(i); });

    enteredSegments
        .append("clipPath")
        .attr("id", clipPathName)
        .append("use")
        .attr("xlink:href", function(d) { return "#" + segmentPathName(d); });

    var imageWidth = 200, imageHeight = 200;
    enteredSegments
        .filter(function(d) { return d.data.placeholderCoverURL; })
        .append("image")
        .attr("clip-path", function(d) { return "url(#" + clipPathName(d) + ")"; })
        .attr("xlink:href", function(d) { return d.data.placeholderCoverURL; })
        .attr("width", 200).attr("height", 200)
        .call(positionImage);

    var enteredLabels =
        labels
        .enter()
        .append("text")
        .attr("text-anchor", "middle")
        .attr("dy", 8) // fudge vertical centering
        .call(positionText)
        .attr("opacity", 0)
        .text(function(d) { return d.data.localizedPluralName; });

    function positionImage(selection) {
        // can't use transform because that is applied _after_ clip path
        selection
            .attr("x", function(d) { return self.arc.centroid(d)[0] - imageWidth / 2; })
            .attr("y", function(d) { return self.arc.centroid(d)[1] - imageHeight / 2; });
    }

    function positionText(selection) {
        selection
            .attr("transform", function(d) { return "translate(" + self.labelArc.centroid(d) + ")"; })
            .attr("opacity", function(d) {
                var active = 1, inactive = 0.4;

                var isTooSmall = (Math.abs(d.endAngle - d.startAngle) < 0.15);
                if (filterMediaType) {
                    if (filterMediaType == d.data.mediaType)
                        return active;
                    return isTooSmall ? 0 : inactive;
                } else {
                    return isTooSmall ? 0 :active;
                }
            });
    }


    // delete
    segments
        .exit()
        .remove();

    labels
        .exit()
        .remove();


    // update (& insert)
    segmentTransitions = 
        segments
        .transition()
        .duration(750);

    segmentTransitions
        .select("path")
        .attrTween("d", arcTween);

    segmentTransitions
        .select("image")
        .call(positionImage);

    segmentTransitions
        .attr("opacity", function(d) {
            return (!filterMediaType || filterMediaType == d.data.mediaType) ? 1.0 : 0.4;
        });

    labels
        .transition()
        .duration(750)
        .call(positionText);

    function arcTween(d) {
        if (this._current === undefined)
            this._current = d;

        var i = d3.interpolate(this._current, d);
        this._current = i(0);
        return function(t) {
            return self.arc(i(t));
        };
    };
}
